package db

import (
	"os"

	"github.com/BurntSushi/toml"
	"github.com/bdlm/log"
	"github.com/google/uuid"
)

type Database struct {
	VAPID    VAPIDPushService                      `toml:"vapid"`
	Sessions map[uuid.UUID]SessionPushNotification `toml:"sessions"`
	Channels map[uuid.UUID]*Channel                `toml:"channels"`
}

func NewDatabase(configPath string) *Database {
	db := &Database{}
	if err := db.read(configPath); err != nil {
		log.Panicf("open config file: %s", err)
	}
	if ok, err := db.VAPID.Generate(); err != nil {
		log.Panicf("generate vapid push: %s", err)
	} else if ok {
		if err = db.save(configPath); err != nil {
			log.Panicf("save new config with vapid push: %s", err)
		} else {
			log.Info("save new config with vapid push")
		}
	} else {
		log.Infof("found vapid push public: %s", db.VAPID.PublicKey)
	}
	if db.Sessions == nil {
		db.Sessions = make(map[uuid.UUID]SessionPushNotification)
	}
	return db
}

func (db *Database) read(path string) error {
	_, err := toml.DecodeFile(path, db)
	return err
}

func (db *Database) save(path string) error {
	tmpFile := path + ".tmp"

	file, err := os.OpenFile(tmpFile, os.O_WRONLY|os.O_CREATE|os.O_TRUNC, 0644)
	if err != nil {
		return err
	}

	err = toml.NewEncoder(file).Encode(db)
	if err != nil {
		return err
	}

	file.Close()
	return os.Rename(tmpFile, path)
}

func (db *Database) SendAll(data interface{}) {
	for _, s := range db.Sessions {
		db.VAPID.SendNotification(s.ToVAPID(), data)
	}
}
